//+------------------------------------------------------------------+
//|                                                 HULL_ANTICIP.mq4 |
//|                                                     Manu29 |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Manu29"
#property link      ""

//#property indicator_separate_window
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red

//---- input parameters
extern int       HullPeriod=16;

//---- buffers
double TempBuf1[];
double HullAntBuf[];
double HullBuf[];

//Variable
int HalfHullPeriod;
int SqrtHullPeriod;

//+------------------------------------------------------------------+
//| Specific functions                                               |
//+------------------------------------------------------------------+
double ComputeHull(double MyArray[], int MyPeriod)
{
   double MyTemp[];
   double result = -1;
   int    i;

   int HalfPeriod = MathRound(MyPeriod/2);
   int SqrtPeriod = MathRound(MathSqrt(MyPeriod));

   //Copy Close Values to CloseTemp
   ArrayResize(MyTemp, SqrtPeriod);
   ArraySetAsSeries(MyTemp, true); 
   
   //HMA value computation
   for(i=0; i<SqrtPeriod; i++)
   {  
       MyTemp[i] = 2*iMAOnArray(MyArray, 0, HalfPeriod, 0, MODE_LWMA, i) - iMAOnArray(MyArray, 0, MyPeriod, 0, MODE_LWMA, i);       
   }
   result = iMAOnArray(MyTemp, 0, SqrtPeriod, 0, MODE_LWMA, 0);

//---- done
   return(result);
}

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- 1 additional buffers are used for temporary data.
   IndicatorBuffers(3);
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
//---- 2 indicator buffers mapping
   SetIndexBuffer(0,HullBuf);
   SetIndexBuffer(1,HullAntBuf);
   SetIndexBuffer(2,TempBuf1);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("HULL ANTICIP("+HullPeriod+")");
   SetIndexLabel(0,"HULL");
   SetIndexLabel(1,"HULL ANTICIPATION");
   SetIndexLabel(2,NULL);

//---- Specific indicator initialization
   HalfHullPeriod = MathRound(HullPeriod/2);
   SqrtHullPeriod = MathRound(MathSqrt(HullPeriod));

//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   double CloseTemp[];
   int bar, limit, i;   
   int counted_bars=IndicatorCounted();
   
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;

   ArrayResize(CloseTemp, HullPeriod+SqrtHullPeriod);
   ArraySetAsSeries(CloseTemp, true); 

   for(i=0; i<limit; i++)
   {
      ArrayCopy(CloseTemp, Close, 0, i, HullPeriod+SqrtHullPeriod);
      HullAntBuf[i]=ComputeHull(CloseTemp, HullPeriod);
   }
   
   //Hull average computation    
   for(i=0; i<limit; i++)
      TempBuf1[i]=2*iMA(NULL,0,HalfHullPeriod,0,MODE_LWMA,PRICE_CLOSE,i)-iMA(NULL,0,HullPeriod,0,MODE_LWMA,PRICE_CLOSE,i);
   for(i=0; i<limit; i++)
      HullBuf[i]=iMAOnArray(TempBuf1,0,SqrtHullPeriod,0,MODE_LWMA,i);

//----
   return(0);
  }
//+------------------------------------------------------------------+